"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var t = __importStar(require("@babel/types"));
var traverse_1 = __importDefault(require("@babel/traverse"));
var parser_1 = require("@babel/parser");
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var resolve_1 = __importDefault(require("resolve"));
/**
 * Returns the nodes path including the scope of a parent.
 * @param node
 * @param parentPath
 */
exports.getPathOfNode = function (node, parentPath) {
    var foundPath = null;
    traverse_1.default(t.expressionStatement(node), {
        enter: function (path) {
            foundPath = path;
            path.stop();
        },
    }, parentPath.scope, undefined, parentPath);
    if (!foundPath) {
        throw parentPath.buildCodeFrameError('No path for a child node was found.');
    }
    return foundPath;
};
/**
 * Builds a code frame error from a passed in node.
 *
 * @param error
 * @param node
 * @param parentPath
 */
exports.buildCodeFrameError = function (error, node, parentPath) {
    var startLoc = node.loc ? " (" + node.loc.start.line + ":" + node.loc.start.column + ")" : '';
    return exports.getPathOfNode(node, parentPath).buildCodeFrameError("" + error + startLoc + ".");
};
/**
 * Returns the binding identifier for a member expression.
 * For example the member expression `foo.bar.baz` will return the `foo` identifier.
 *
 * @param expression - Member expression node.
 */
exports.getMemberExpressionMeta = function (expression) {
    var bindingIdentifier = null;
    var accessPath = [];
    if (t.isIdentifier(expression.property)) {
        accessPath.push(expression.property);
    }
    traverse_1.default(t.expressionStatement(expression), {
        noScope: true,
        MemberExpression: function (path) {
            if (t.isIdentifier(path.node.object)) {
                bindingIdentifier = path.node.object;
            }
            if (t.isIdentifier(path.node.property)) {
                accessPath.push(path.node.property);
            }
        },
    });
    return {
        bindingIdentifier: bindingIdentifier,
        accessPath: accessPath.reverse(),
    };
};
/**
 * Will return the value of a path from an object expression.
 *
 * For example if  we take an object expression that looks like:
 * ```
 * { colors: { primary: 'red' } }
 * ```
 *
 * And a path of identifiers that looks like:
 * ```
 * [colors, primary]
 * ```
 *
 * Would result in returning the `red` string literal node.
 * If the value is not found `undefined` will be returned.
 *
 * @param expression - Member expression node.
 * @param accessPath - Access path identifiers.
 */
exports.getValueFromObjectExpression = function (expression, accessPath) {
    var value = undefined;
    traverse_1.default(expression, {
        noScope: true,
        ObjectProperty: function (path) {
            if (t.isIdentifier(path.node.key, { name: accessPath[0].name })) {
                if (t.isObjectExpression(path.node.value)) {
                    value = exports.getValueFromObjectExpression(path.node.value, accessPath.slice(1));
                }
                else {
                    value = path.node.value;
                }
                path.stop();
            }
        },
    });
    return value;
};
/**
 * Will return either the name of an identifier or the value of a string literal.
 *
 * E.g:
 * - `foo` identifier node will return `"foo"`,
 * - `"bar"` string literal node will return `"bar"`.
 *
 * @param node
 */
exports.getKey = function (node) {
    if (t.isIdentifier(node)) {
        return node.name;
    }
    if (t.isStringLiteral(node)) {
        return node.value;
    }
    throw new Error(node.type + " has no name.'");
};
/**
 * Returns `true` if an identifier or any paths that reference the identifier are mutated.
 * @param path
 */
var isIdentifierReferencesMutated = function (path) {
    var binding = path.scope.getBinding(path.node.name);
    if (!binding) {
        return false;
    }
    if (!t.isVariableDeclarator(binding.path.node) || !binding.constant) {
        return true;
    }
    for (var i = 0; i < binding.referencePaths.length; i++) {
        var refPath = binding.referencePaths[i];
        var innerBinding = refPath.scope.getBinding(path.node.name);
        if (!innerBinding) {
            continue;
        }
        if (!t.isVariableDeclarator(innerBinding.path.node) || !innerBinding.constant) {
            return true;
        }
    }
    return false;
};
/**
 * Will traverse a path and its identifiers to find all bindings.
 * If any of those bindings are mutated `true` will be returned.
 *
 * @param path
 */
exports.isPathReferencingAnyMutatedIdentifiers = function (path) {
    if (path.isIdentifier()) {
        return isIdentifierReferencesMutated(path);
    }
    var mutated = false;
    path.traverse({
        Identifier: function (innerPath) {
            var result = isIdentifierReferencesMutated(path);
            if (result) {
                mutated = true;
                // No need to keep traversing - let's stop!
                innerPath.stop();
            }
        },
    });
    return mutated;
};
/**
 * Will try to statically evaluate the node.
 * If successful it will return a literal node,
 * else it will return the fallback node.
 *
 * @param node Node to evaluate
 * @param meta
 * @param fallbackNode Optional node to return if evaluation is not successful. Defaults to `node`.
 */
exports.tryEvaluateExpression = function (node, meta, fallbackNode) {
    if (fallbackNode === void 0) { fallbackNode = node; }
    if (!node) {
        return node;
    }
    var path = exports.getPathOfNode(node, meta.parentPath);
    if (exports.isPathReferencingAnyMutatedIdentifiers(path)) {
        return fallbackNode;
    }
    var result = path.evaluate();
    if (result.value) {
        switch (typeof result.value) {
            case 'string':
                return t.stringLiteral(result.value);
            case 'number':
                return t.numericLiteral(result.value);
        }
    }
    return fallbackNode;
};
/**
 * Will return the `node` of the a binding.
 * This function will follow import specifiers to return the actual `node`.
 *
 * When wanting to do futher traversal on the resulting `node` make sure to use the output `meta` as well.
 * The `meta` will be for the resulting file it was found in.
 *
 * @param path
 */
exports.resolveBindingNode = function (binding, meta) {
    if (!binding || binding.path.isObjectPattern()) {
        // Bail early if there is no binding or its a node that we don't want to resolve
        // such as an destructured args from a function.
        return undefined;
    }
    if (t.isVariableDeclarator(binding.path.node)) {
        return {
            meta: meta,
            node: binding.path.node.init,
            path: binding.path,
            constant: binding.constant,
            source: 'module',
        };
    }
    if (binding.path.parentPath.isImportDeclaration()) {
        if (!meta.state.filename) {
            throw new Error('Filename is needed for module traversal.');
        }
        var moduleImportName = binding.path.parentPath.node.source.value;
        var isRelative = moduleImportName.charAt(0) === '.';
        var filename = isRelative
            ? path_1.default.join(path_1.default.dirname(meta.state.filename), moduleImportName)
            : moduleImportName;
        var modulePath = resolve_1.default.sync(filename, {
            extensions: ['.js', '.jsx', '.ts', '.tsx'],
        });
        var moduleCode = fs_1.default.readFileSync(modulePath, 'utf-8');
        var ast = parser_1.parse(moduleCode, { sourceType: 'module', sourceFilename: modulePath });
        var foundNode_1 = undefined;
        var foundParentPath_1 = undefined;
        if (binding.path.isImportDefaultSpecifier()) {
            traverse_1.default(ast, {
                ExportDefaultDeclaration: function (path) {
                    foundParentPath_1 = path;
                    foundNode_1 = path.node.declaration;
                },
            });
        }
        else if (binding.path.isImportSpecifier()) {
            var exportName_1 = binding.path.node.local.name;
            traverse_1.default(ast, {
                ExportNamedDeclaration: function (path) {
                    if (!path.node.declaration || !t.isVariableDeclaration(path.node.declaration)) {
                        return;
                    }
                    for (var i = 0; i < path.node.declaration.declarations.length; i++) {
                        var named = path.node.declaration.declarations[i];
                        if (t.isIdentifier(named.id) && named.id.name === exportName_1) {
                            foundNode_1 = named.init;
                            foundParentPath_1 = path;
                            path.stop();
                            break;
                        }
                    }
                },
            });
        }
        if (!foundNode_1 || !foundParentPath_1) {
            return undefined;
        }
        return {
            constant: binding.constant,
            node: foundNode_1,
            path: foundParentPath_1,
            source: 'import',
            meta: __assign(__assign({}, meta), { parentPath: foundParentPath_1, state: __assign(__assign({}, meta.state), { file: ast, filename: filename }) }),
        };
    }
    return {
        node: binding.path.node,
        path: binding.path,
        constant: binding.constant,
        source: 'module',
        meta: meta,
    };
};
/**
 * Will wrap BlockStatement or Expression in an IIFE,
 * Looks like (() => { return 10; })().
 *
 * @param node Node of type either BlockStatement or Expression
 */
exports.wrapNodeInIIFE = function (node) {
    return t.callExpression(t.arrowFunctionExpression([], node), []);
};
var tryWrappingBlockStatementInIIFE = function (node) {
    return t.isBlockStatement(node) ? exports.wrapNodeInIIFE(node) : node;
};
/**
 * Will pick `Function` body and tries to wrap it in an IIFE if
 * its a BlockStatement otherwise returns the picked body,
 * E.g.
 * `props => props.color` would end up as `props.color`.
 * `props => { return props.color; }` would end up as `(() => { return props.color })()`
 * `function () { return props.color; }` would end up as `(function () { return props.color })()`
 *
 * @param node Node of type ArrowFunctionExpression
 */
exports.pickFunctionBody = function (node) { return tryWrappingBlockStatementInIIFE(node.body); };
//# sourceMappingURL=ast.js.map